@@ -31,6 +31,7 @@ from .._protos.build.bazel.remote.asset.v1 import remote_asset_pb2_grpc
 from .._protos.build.bazel.remote.execution.v2 import remote_execution_pb2_grpc
 from .._protos.build.buildgrid import local_cas_pb2_grpc
 from .._protos.google.bytestream import bytestream_pb2_grpc
+from .._protos.google.longrunning import operations_pb2_grpc
 
 from .. import _site
 from .. import utils
@@ -57,12 +58,27 @@ _REQUIRED_CASD_MICRO = 58
 #     log_dir (str): The directory for the logs
 #     log_level (LogLevel): Log level to give to buildbox-casd for logging
 #     cache_quota (int): User configured cache quota
+#     reserved (int): User configured reserved disk space
 #     remote_cache_spec (RemoteSpec): Optional remote cache server
 #     protect_session_blobs (bool): Disable expiry for blobs used in the current session
 #     messenger (Messenger): The messenger to report warnings through the UI
 #
 class CASDProcessManager:
-    def __init__(self, path, log_dir, log_level, cache_quota, remote_cache_spec, protect_session_blobs, messenger):
+    def __init__(
+        self,
+        path,
+        log_dir,
+        log_level,
+        cache_quota,
+        remote_cache_spec,
+        protect_session_blobs,
+        messenger,
+        *,
+        reserved=None,
+        low_watermark=None
+    ):
+        os.makedirs(path, exist_ok=True)
+
         self._log_dir = log_dir
 
         self._socket_path = self._make_socket_path(path)
@@ -77,7 +93,12 @@ class CASDProcessManager:
 
         if cache_quota is not None:
             casd_args.append("--quota-high={}".format(int(cache_quota)))
-            casd_args.append("--quota-low={}".format(int(cache_quota / 2)))
+
+        if low_watermark is not None:
+            casd_args.append("--quota-low={}%".format(int(low_watermark * 100)))
+
+        if reserved is not None:
+            casd_args.append("--reserved={}".format(int(reserved)))
 
         if protect_session_blobs:
             casd_args.append("--protect-session-blobs")
@@ -91,6 +112,14 @@ class CASDProcessManager:
             if remote_cache_spec.client_key_file:
                 casd_args.append("--cas-client-key={}".format(remote_cache_spec.client_key_file))
                 casd_args.append("--cas-client-cert={}".format(remote_cache_spec.client_cert_file))
+            if remote_cache_spec.access_token_file:
+                casd_args.append("--cas-access-token={}".format(remote_cache_spec.access_token_file))
+                if remote_cache_spec.access_token_reload_interval is not None:
+                    casd_args.append(
+                        "--cas-token-reload-interval={}".format(remote_cache_spec.access_token_reload_interval)
+                    )
+            if remote_cache_spec.keepalive_time is not None:
+                casd_args.append("--cas-keepalive-time={}".format(remote_cache_spec.keepalive_time))
 
         casd_args.append(path)
 
@@ -109,6 +138,19 @@ class CASDProcessManager:
                 env=self.__buildbox_casd_env(),
             )
 
+        self._casd_channel = None
+        self._bytestream = None
+        self._casd_cas = None
+        self._local_cas = None
+        self._asset_fetch = None
+        self._asset_push = None
+        self._exec_service = None
+        self._operations_service = None
+        self._ac_service = None
+        self._shutdown_requested = False
+
+        self._lock = threading.Lock()
+
     def __buildbox_casd(self):
         return utils._get_host_tool_internal("buildbox-casd", search_subprojects_dir="buildbox")
 
@@ -259,6 +301,20 @@ class CASDProcessManager:
     # Terminate the process and release related resources.
     #
     def release_resources(self, messenger=None):
+        self._shutdown_requested = True
+        with self._lock:
+            if self._casd_channel:
+                self._ac_service = None
+                self._operations_service = None
+                self._exec_service = None
+                self._asset_push = None
+                self._asset_fetch = None
+                self._local_cas = None
+                self._casd_cas = None
+                self._bytestream = None
+                self._casd_channel.close()
+                self._casd_channel = None
+
         self._terminate(messenger)
         self.process = None
         shutil.rmtree(self._socket_tempdir)
@@ -305,31 +361,6 @@ class CASDProcessManager:
                 "Buildbox-casd didn't exit cleanly. Exit code: {}, Logs: {}".format(return_code, self._logfile)
             )
 
-    # create_channel():
-    #
-    # Return a CASDChannel, note that the actual connection is not necessarily
-    # established until it is needed.
-    #
-    def create_channel(self):
-        return CASDChannel(self._socket_path, self._connection_string, self._start_time, self.process.pid)
-
-
-class CASDChannel:
-    def __init__(self, socket_path, connection_string, start_time, casd_pid):
-        self._socket_path = socket_path
-        self._connection_string = connection_string
-        self._start_time = start_time
-        self._casd_channel = None
-        self._bytestream = None
-        self._casd_cas = None
-        self._local_cas = None
-        self._asset_fetch = None
-        self._asset_push = None
-        self._casd_pid = casd_pid
-        self._shutdown_requested = False
-
-        self._lock = threading.Lock()
-
     def _establish_connection(self):
         with self._lock:
             if self._casd_channel is not None:
@@ -347,7 +378,7 @@ class CASDChannel:
 
                 # check that process is still alive
                 try:
-                    proc = psutil.Process(self._casd_pid)
+                    proc = psutil.Process(self.process.pid)
                     if proc.status() == psutil.STATUS_ZOMBIE:
                         proc.wait()
 
@@ -368,6 +399,9 @@ class CASDChannel:
             self._local_cas = local_cas_pb2_grpc.LocalContentAddressableStorageStub(self._casd_channel)
             self._asset_fetch = remote_asset_pb2_grpc.FetchStub(self._casd_channel)
             self._asset_push = remote_asset_pb2_grpc.PushStub(self._casd_channel)
+            self._exec_service = remote_execution_pb2_grpc.ExecutionStub(self._casd_channel)
+            self._operations_service = operations_pb2_grpc.OperationsStub(self._casd_channel)
+            self._ac_service = remote_execution_pb2_grpc.ActionCacheStub(self._casd_channel)
 
     # get_cas():
     #
@@ -410,37 +444,29 @@ class CASDChannel:
             self._establish_connection()
         return self._asset_push
 
-    # is_closed():
+    # get_exec_service():
     #
-    # Return whether this connection is closed or not.
+    # Return Remote Execution stub for buildbox-casd channel.
     #
-    def is_closed(self):
-        return self._casd_channel is None
+    def get_exec_service(self):
+        if self._casd_channel is None:
+            self._establish_connection()
+        return self._exec_service
 
-    # request_shutdown():
+    # get_operations_service():
     #
-    # Notify the channel that a shutdown of casd was requested.
+    # Return Operations stub for buildbox-casd channel.
     #
-    # Thus we know that not being able to establish a connection is expected
-    # and no error will be reported in that case.
-    def request_shutdown(self) -> None:
-        self._shutdown_requested = True
+    def get_operations_service(self):
+        if self._casd_channel is None:
+            self._establish_connection()
+        return self._operations_service
 
-    # close():
+    # get_ac_service():
     #
-    # Close the casd channel.
+    # Return Action Cache stub for buildbox-casd channel.
     #
-    def close(self):
-        assert self._shutdown_requested, "Please request shutdown before closing"
-
-        with self._lock:
-            if self.is_closed():
-                return
-
-            self._asset_push = None
-            self._asset_fetch = None
-            self._local_cas = None
-            self._casd_cas = None
-            self._bytestream = None
-            self._casd_channel.close()
-            self._casd_channel = None
+    def get_ac_service(self):
+        if self._casd_channel is None:
+            self._establish_connection()
+        return self._ac_service
