@@ -351,9 +351,13 @@ class LogLine(Widget):
             key = element._get_display_key()
             dim_keys = not key.strict
 
+            # Guarantee that description is reported on a single line.
+            description = " ".join(element._description.splitlines())
+
             line = p.fmt_subst(line, "name", element._get_full_name(), fg="blue", bold=True)
             line = p.fmt_subst(line, "key", key.brief, fg="yellow", dim=dim_keys)
             line = p.fmt_subst(line, "full-key", key.full, fg="yellow", dim=dim_keys)
+            line = p.fmt_subst(line, "description", description, fg="yellow", dim=dim_keys)
 
             try:
                 if not element._has_all_sources_resolved():
@@ -1036,7 +1040,7 @@ class LogLine(Widget):
                     + "{}".format(filename)
                 )
             elif filestat.file_type == FileType.SYMLINK:
-                target = directory.readlink(*filename.split(os.path.sep))
+                target = directory.readlink(filename)
                 return (
                     "lrwxrwxrwx  link   {}".format(size)
                     + "{} ".format(" " * (max_v_len - len(size)))
