@@ -56,7 +56,7 @@ _PROJECT_CONF_FILE = "project.conf"
 class ProjectConfig:
     def __init__(self):
         self.options = None  # OptionPool
-        self.base_variables = {}  # The base set of variables
+        self.base_variables = None  # The base set of variables
         self.element_overrides = {}  # Element specific configurations
         self.source_overrides = {}  # Source specific configurations
         self.mirrors = {}  # Dictionary of SourceMirror objects
@@ -90,6 +90,7 @@ class Project:
         parent_loader: Optional[Loader] = None,
         provenance_node: Optional[ProvenanceInformation] = None,
         search_for_project: bool = True,
+        fetch_subprojects=None
     ):
         #
         # Public members
@@ -161,6 +162,7 @@ class Project:
             self.load_context = parent_loader.load_context
         else:
             self.load_context = LoadContext(self._context)
+            self.load_context.set_fetch_subprojects(fetch_subprojects)
 
         if search_for_project:
             self.directory, self._invoked_from_workspace_element = self._find_project_dir(directory)
@@ -543,6 +545,15 @@ class Project:
             return
         assert self._partially_loaded
 
+        # Ensure that the parent project is fully loaded first
+        if self.junction:
+            self.junction._get_project().ensure_fully_loaded()
+
+            if self._fully_loaded:
+                # The parent project triggered the loading of this project,
+                # don't load this subproject twice.
+                return
+
         # Here we mark the project as fully loaded right away,
         # before doing the work.
         #
@@ -558,9 +569,6 @@ class Project:
         #
         self._fully_loaded = True
 
-        if self.junction:
-            self.junction._get_project().ensure_fully_loaded()
-
         self._load_second_pass()
 
     # get_default_target()
@@ -1100,6 +1108,9 @@ class Project:
         # Override default_mirror if not set by command-line
         output.default_mirror = self._default_mirror or overrides.get_str("default-mirror", default=None)
 
+        # Source url aliases
+        output._aliases = config.get_mapping("aliases", default={})
+
         # First try mirrors specified in user configuration, user configuration
         # is allowed to completely disable mirrors by specifying an empty list,
         # so we check for a None value here too.
@@ -1121,9 +1132,6 @@ class Project:
             if not output.default_mirror:
                 output.default_mirror = mirror.name
 
-        # Source url aliases
-        output._aliases = config.get_mapping("aliases", default={})
-
         # Perform variable substitutions in source aliases
         variables.expand(output._aliases)
 
@@ -1151,9 +1159,8 @@ class Project:
         elif filename == WORKSPACE_PROJECT_FILE:
             workspace_project_cache = self._context.get_workspace_project_cache()
             workspace_project = workspace_project_cache.get(found_directory)
-            if workspace_project:
-                project_directory = workspace_project.get_default_project_path()
-                workspace_element = workspace_project.get_default_element()
+            project_directory = workspace_project.get_default_project_path()
+            workspace_element = workspace_project.get_default_element()
         else:
             raise LoadError(
                 "None of {names} found in '{path}' or any of its parent directories".format(
